@@ -30,7 +30,7 @@ from pkg_resources import (
 import setuptools.unicode_utils as unicode_utils
 from setuptools.glob import glob
 
-from pkg_resources.extern import packaging
+from setuptools.extern import packaging
 
 
 def translate_pattern(glob):
@@ -112,10 +112,37 @@ def translate_pattern(glob):
         if not last_chunk:
             pat += sep
 
-    return re.compile(pat + r'\Z(?ms)')
+    pat += r'\Z'
+    return re.compile(pat, flags=re.MULTILINE|re.DOTALL)
 
 
-class egg_info(Command):
+class InfoCommon:
+    tag_build = None
+    tag_date = None
+
+    @property
+    def name(self):
+        return safe_name(self.distribution.get_name())
+
+    def tagged_version(self):
+        version = self.distribution.get_version()
+        # egg_info may be called more than once for a distribution,
+        # in which case the version string already contains all tags.
+        if self.vtags and version.endswith(self.vtags):
+            return safe_version(version)
+        return safe_version(version + self.vtags)
+
+    def tags(self):
+        version = ''
+        if self.tag_build:
+            version += self.tag_build
+        if self.tag_date:
+            version += time.strftime("-%Y%m%d")
+        return version
+    vtags = property(tags)
+
+
+class egg_info(InfoCommon, Command):
     description = "create a distribution's .egg-info directory"
 
     user_options = [
@@ -132,14 +159,11 @@ class egg_info(Command):
     }
 
     def initialize_options(self):
-        self.egg_name = None
-        self.egg_version = None
         self.egg_base = None
+        self.egg_name = None
         self.egg_info = None
-        self.tag_build = None
-        self.tag_date = 0
+        self.egg_version = None
         self.broken_egg_info = False
-        self.vtags = None
 
     ####################################
     # allow the 'tag_svn_revision' to be detected and
@@ -159,9 +183,7 @@ class egg_info(Command):
         build tag. Install build keys in a deterministic order
         to avoid arbitrary reordering on subsequent builds.
         """
-        # python 2.6 compatibility
-        odict = getattr(collections, 'OrderedDict', dict)
-        egg_info = odict()
+        egg_info = collections.OrderedDict()
         # follow the order these keys would have been added
         # when PYTHONHASHSEED=0
         egg_info['tag_build'] = self.tags()
@@ -169,10 +191,12 @@ class egg_info(Command):
         edit_config(filename, dict(egg_info=egg_info))
 
     def finalize_options(self):
-        self.egg_name = safe_name(self.distribution.get_name())
-        self.vtags = self.tags()
+        # Note: we need to capture the current value returned
+        # by `self.tagged_version()`, so we can later update
+        # `self.distribution.metadata.version` without
+        # repercussions.
+        self.egg_name = self.name
         self.egg_version = self.tagged_version()
-
         parsed_version = parse_version(self.egg_version)
 
         try:
@@ -255,14 +279,6 @@ class egg_info(Command):
         if not self.dry_run:
             os.unlink(filename)
 
-    def tagged_version(self):
-        version = self.distribution.get_version()
-        # egg_info may be called more than once for a distribution,
-        # in which case the version string already contains all tags.
-        if self.vtags and version.endswith(self.vtags):
-            return safe_version(version)
-        return safe_version(version + self.vtags)
-
     def run(self):
         self.mkpath(self.egg_info)
         installer = self.distribution.fetch_build_egg
@@ -278,14 +294,6 @@ class egg_info(Command):
 
         self.find_sources()
 
-    def tags(self):
-        version = ''
-        if self.tag_build:
-            version += self.tag_build
-        if self.tag_date:
-            version += time.strftime("-%Y%m%d")
-        return version
-
     def find_sources(self):
         """Generate SOURCES.txt manifest file"""
         manifest_filename = os.path.join(self.egg_info, "SOURCES.txt")
@@ -429,7 +437,11 @@ class FileList(_FileList):
 
     def graft(self, dir):
         """Include all files from 'dir/'."""
-        found = distutils.filelist.findall(dir)
+        found = [
+            item
+            for match_dir in glob(dir)
+            for item in distutils.filelist.findall(match_dir)
+        ]
         self.extend(found)
         return bool(found)
 
@@ -594,6 +606,7 @@ def write_pkg_info(cmd, basename, filename):
         metadata = cmd.distribution.metadata
         metadata.version, oldver = cmd.egg_version, metadata.version
         metadata.name, oldname = cmd.egg_name, metadata.name
+
         try:
             # write unescaped data to PKG-INFO, so older pkg_resources
             # can still parse it
@@ -633,7 +646,7 @@ def write_requirements(cmd, basename, filename):
 
 
 def write_setup_requirements(cmd, basename, filename):
-    data = StringIO()
+    data = io.StringIO()
     _write_requirements(data, cmd.distribution.setup_requires)
     cmd.write_or_delete_file("setup-requirements", filename, data.getvalue())
 
